package com.linkv.rtcsdk.lvrtc;

import android.app.Application;
import android.text.TextUtils;
import android.view.ViewGroup;

import com.linkv.rtc.callback.LVResultCallback1;
import com.linkv.rtc.callback.LVResultCallback2;
import com.linkv.rtcsdk.LinkVRTCEngine;
import com.linkv.rtcsdk.bean.VideoQuality;
import com.linkv.rtcsdk.utils.LogUtils;
import com.linkv.rtc.LVConstants;
import com.linkv.rtc.LVErrorCode;
import com.linkv.rtc.LVRTCEngine;
import com.linkv.rtc.callback.LVRTCCallback;
import com.linkv.rtc.entity.LVAVConfig;
import com.linkv.rtc.entity.LVAudioVolume;
import com.linkv.rtc.entity.LVUser;
import com.linkv.rtc.entity.LVVideoStatistic;
import com.linkv.rtc.render.LVDisplayView;

import java.nio.ByteBuffer;
import java.util.ArrayList;

/**
 * Created by Xiaohong on 2020/7/18.
 * desc: 自研RTC实现类
 */
public class LVRTCLive extends LinkVRTCEngine {
    private static final String TAG = "LVRTCLive";
    private com.linkv.rtc.LVRTCEngine mLVRTCEngine;
    private volatile boolean isInited = false;
    private Application mApplication;
    // 默认分辨率640*360
    private LVAVConfig mLVAVConfig = new LVAVConfig(LVAVConfig.PROFILE_360P);

    public LVRTCLive() {
        super();
        this.sdk_type = SDK_TYPE_LVRTC;
    }


    @Override
    public void setLogOpen(boolean isOpen) {
        super.setLogOpen(isOpen);
        LVRTCEngine.setLogLevel(isOpen ? LVConstants.LogLevel.LS_VERBOSE : LVConstants.LogLevel.LS_NONE);
    }

    @Override
    public void setVideoConfig(VideoConfigLevel videoConfigLevel) {
        switch (videoConfigLevel) {
            case VIDEO_CONFIG_180P:
                mLVAVConfig = new LVAVConfig(LVAVConfig.PROFILE_180P);
                break;
            case VIDEO_CONFIG_270P:
                mLVAVConfig = new LVAVConfig(LVAVConfig.PROFILE_270P);
                break;
            case VIDEO_CONFIG_360P:
                mLVAVConfig = new LVAVConfig(LVAVConfig.PROFILE_360P);
                break;
            case VIDEO_CONFIG_540P:
                mLVAVConfig = new LVAVConfig(LVAVConfig.PROFILE_540P);
                break;
            case VIDEO_CONFIG_720P:
            case VIDEO_CONFIG_1080P:
                // 自研没有1080p，所以设为720p
                mLVAVConfig = new LVAVConfig(LVAVConfig.PROFILE_720P);
                break;
        }
    }

    @Override
    protected void initSDK(Application application, String appID, String appKey, boolean isDebugEnv, boolean isInternationalEnv, final IInitHandler iInitHandler) {
        this.mApplication = application;
        mLVRTCEngine = LVRTCEngine.getInstance(mApplication.getApplicationContext());
        LVRTCEngine.setUseInternationalEnv(isInternationalEnv);
        LVRTCEngine.setUseTestEnv(isDebugEnv);
        // 必须设置，否则拉流不成功
        mLVRTCEngine.setAVConfig(mLVAVConfig);
        mLVRTCEngine.initSDK();
        if (!isInited) {
            mLVRTCEngine.auth(appID, appKey, mUid, new LVResultCallback1<Integer>() {
                @Override
                public void onResult(Integer integer) {
                    LogUtils.d(TAG, " initSDK   result = " + integer);
                    isInited = integer == LVErrorCode.SUCCESS;
                    iInitHandler.onInitResult(integer);
                }
            });
        } else {
            LogUtils.d(TAG, "已经执行过初始化了。");
            iInitHandler.onInitResult(IInitHandler.INIT_RESULT_SUCCEED);
        }
    }

    @Override
    public void loginRoom(String uid, String roomId, boolean isHost, boolean isAudioOnly) {
        this.mRoomId = roomId;
        this.mIsHost = isHost;
        this.mUid = uid;
        LogUtils.d(TAG, "loginRoom.  uid = " + uid + ", mIsHost=" + mIsHost);
        // 设置房间事件监听
        setLiveRoomCallback();
        if (mLVRTCEngine == null) {
            LogUtils.e(TAG, "RTC 引擎未初始化，请先调用initSDK方法");
            return;
        }

        mLVRTCEngine.loginRoom(mRoomId, mUid, mIsHost, isAudioOnly, new LVResultCallback2<Integer, ArrayList<LVUser>>() {
            @Override
            public void onResult(Integer errorCode, ArrayList<LVUser> users) {
                LogUtils.d(TAG, "onLoginCompletion errorCode:" + errorCode + " mRoomId:" + mRoomId);
                if (errorCode == LVErrorCode.SUCCESS) {
                    if (!mIsHost) {
                        if (users.size() <= 0) {
                            return;
                        }
                        for (LVUser user : users) {
                            String userId = user.userId;
                            LogUtils.d(TAG, "loginRoom userId = " + userId);
                        }
                    }
                    onLoginRoomSucceed(mRoomId);
                } else {
                    onExitRoom(errorCode, mRoomId);
                }
                LogUtils.d(TAG, "login. end");
            }
        });
    }

    @Override
    protected String getSolutionName() {
        return TAG;
    }

    @Override
    public String getRoomId() {
        return mRoomId;
    }

    @Override
    public int getSupportLine() {
        // 支持连麦数8
        return 8;
    }

    @Override
    public int getSDKType() {
        return sdk_type;
    }

    @Override
    public void logoutRoom() {
        LogUtils.d(TAG, "logoutRoom");
        super.logoutRoom();
        if (mLVRTCEngine != null) {
            mLVRTCEngine.logoutRoom(null);
            mLVRTCEngine.setLiveRoomCallback(null);
        }
    }

    @Override
    public void startPreview(String userId, ViewGroup container, boolean isZOrderMediaOverlay) {
        this.mUid = userId;
        if (mLVRTCEngine != null) {
            mLVRTCEngine.setAVConfig(mLVAVConfig);
            LVDisplayView localPreview = new LVDisplayView()
                    .setUid(userId)
                    .isLocalPreviewView(true)
                    .setLayoutContainer(container)
                    .isZOrderMediaOverlay(isZOrderMediaOverlay);
            mLVRTCEngine.addDisplayView(mApplication, localPreview);
            mLVRTCEngine.startCapture();
        }
    }

    @Override
    public void startPublishing() {
        if (mLVRTCEngine != null) {
            mLVRTCEngine.setAVConfig(mLVAVConfig);
            mLVRTCEngine.startPublishing();
        }
    }


    @Override
    public void setPublishQualityMonitorCycle(int second) {
        if (mLVRTCEngine != null) {
            mLVRTCEngine.setPublishQualityMonitorCycle(second);
        }
    }

    @Override
    public void setPlayQualityMonitorCycle(int second) {
        if (mLVRTCEngine != null) {
            mLVRTCEngine.setPlayQualityMonitorCycle(second);
        }
    }

    @Override
    public void startPlayingStream(String userId, ViewGroup container, boolean isZOrderMediaOverlay) {
        // 视频流渲染视图用户ID进行绑定
        if (mLVRTCEngine != null) {
            LVDisplayView lvDisplayView = new LVDisplayView()
                    .setUid(userId)
                    .isLocalPreviewView(false)
                    .setLayoutContainer(container)
                    .isZOrderMediaOverlay(isZOrderMediaOverlay);
            mLVRTCEngine.addDisplayView(mApplication, lvDisplayView);
            // 播放远端流
            mLVRTCEngine.startPlayingStream(userId);
        }
    }

    @Override
    public void useFrontCamera(boolean mFrontCamera) {
        if (mLVRTCEngine != null) {
            mLVRTCEngine.switchCamera(mFrontCamera ? LVConstants.CMRTCCameraPosition.FRONT : LVConstants.CMRTCCameraPosition.BACK);
        }
    }

    @Override
    public void startCapture() {
        if (mLVRTCEngine != null) {
            mLVRTCEngine.startCapture();
        }
    }

    @Override
    public void stopCapture() {
        if (mLVRTCEngine != null) {
            mLVRTCEngine.stopCapture();
        }
    }

    @Override
    public void stopPreview() {
        if (mLVRTCEngine != null) {
            mLVRTCEngine.stopCapture();
            mLVRTCEngine.removeDisplayView(mUid);
        }
    }

    @Override
    public void stopPublishingStream() {
        if (mLVRTCEngine != null) {
            mLVRTCEngine.stopPublishing();
        }

    }

    @Override
    public void stopPlayingStream(String userId) {
        if (mLVRTCEngine != null) {
            mLVRTCEngine.stopPlayingStream(userId);
            mLVRTCEngine.removeDisplayView(userId);
        }
    }

    @Override
    public void setMuteOutput(boolean mute) {
        LogUtils.d(TAG, "setMuteOutput mute:" + mute);
        if (mLVRTCEngine != null) {
            mLVRTCEngine.enableMic(!mute);
        }
    }

    // 设置房间事件回调
    private void setLiveRoomCallback() {
        if (mLVRTCEngine != null) {
            mLVRTCEngine.setLiveRoomCallback(new LVRTCCallback() {
                private int mLastFrameWidth;
                private int mLastFrameHeight;

                @Override
                public long onDrawFrame(ByteBuffer i420Buffer, final int width, final int height, int strideY, final String userId, final String ext) {
                    // 收到远端视频帧数据回调
                    return 0;
                }

                @Override
                public void onAddRemoter(LVUser user) {
                    // 有推流成员加入的通知
                    String userId = user.userId;
                    String[] pullUrls = user.pullUrls;
                    LogUtils.d(TAG, "onAddRemoter userId: " + userId);
                    if (TextUtils.isEmpty(userId) || userId.equals(mUid)) return;
                    onAddRemoterStream(userId);
                }

                @Override
                public void onDeleteRemoter(final String userId) {
                    // 远端推流用户离开房间。
                    LogUtils.d(TAG, "onDeleteRemoter userId: " + userId);
                    if (TextUtils.isEmpty(userId)) return;
                    onRemoteStreamLeave(userId);
                }

                @Override
                public void onKickOff(int reason, String roomId) {
                    // 被踢出房间
                    if (!roomId.equals(mRoomId)) {
                        LogUtils.d(TAG, "onKickOff reason: " + reason + ", roomId: " + roomId + ", current roomId: " + mRoomId);
                        return;
                    }
                    LogUtils.d(TAG, "onKickOff reason: " + reason + ", roomId: " + roomId);
                    LVRTCLive.this.onExitRoom(reason, roomId);
                }

                @Override
                public void onRoomDisconnected(final int errorCode) {
                    // 房间断开的通知
                    LogUtils.d(TAG, "onRoomDisconnect errorCode:" + errorCode + "  mRoomId:" + mRoomId);
                    LVRTCLive.this.onExitRoom(errorCode, mRoomId);
                }

                @Override
                public void onRoomReconnected() {
                    LogUtils.d(TAG, "onRoomReconnected");
                }

                @Override
                public void onMixComplete(boolean success) {
                }

                @Override
                public void onPublishStateUpdate(final int state) {
                    LogUtils.d(TAG, "onPublishStateUpdate " + " stateCode:" + state);
                    invokePublishStateUpdate(state);
                }

                @Override
                public void onPublishQualityUpdate(LVVideoStatistic stats) {
                    VideoQuality videoQuality = new VideoQuality(stats);
                    onPublishStreamQualityUpdate(mUid, videoQuality);
                }

                @Override
                public void onPlayQualityUpdate(final LVVideoStatistic stats, final String userId) {
                    // 播放质量变化的通知
                    VideoQuality videoQuality = new VideoQuality(stats);
                    onRemoteQualityUpdate(userId, videoQuality);
                    int frameWidth = stats.frameWidth;
                    int frameHeight = stats.frameHeight;
                    if (mLastFrameWidth != frameWidth || mLastFrameHeight != frameHeight) {
                        onVideoSizeChanged(userId, frameWidth, frameHeight);
                        mLastFrameWidth = frameWidth;
                        mLastFrameHeight = frameHeight;
                    }
                }

                @Override
                public void onPlayStateUpdate(int state, String userId) {
                    LogUtils.d(TAG, "onPlayStateUpdate " + " stateCode:" + state);
                    invokePlayStateUpdate(state, userId);
                }

                @Override
                public void onAudioMixStream(byte[] audioBuffer, int sampleRate, int channels, int samplesPerChannel,
                                             int bitsPerSample, LVConstants.AudioRecordType type) {
//                    混音录音数据回掉
                }

                @Override
                public void onAudioVolumeUpdate(ArrayList<LVAudioVolume> volumes) {
                    // 音量变化回调
                }

                @Override
                public void onExitRoomComplete() {
                    LogUtils.d(TAG, "onExitRoomComplete ");
                }

                @Override
                public String onMediaSideInfoInPublishVideoFrame() {
                    return null;
                }

                public void onError(int error) {
                }
            });
        }
    }


    @Override
    public void unInitSDK() {
        if (mLVRTCEngine != null) {
            mLVRTCEngine.unInitSDK();
        }
    }


    /**
     * 设置滤镜美颜级别
     *
     * @param beautyLevel 滤镜美颜级别
     */
    public void setBeautyLevel(float beautyLevel) {
        if (mLVRTCEngine != null) {
            mLVRTCEngine.getLVBeautyManager().setBeautyLevel(beautyLevel);
        }
    }

    /**
     * 设置滤镜明亮度
     *
     * @param brightLevel 滤镜明亮度
     */
    public void setBrightLevel(float brightLevel) {
        if (mLVRTCEngine != null) {
            mLVRTCEngine.getLVBeautyManager().setBrightLevel(brightLevel);
        }
    }

    /**
     * 设置滤镜饱和度
     *
     * @param toneLevel 滤镜饱和度级别
     */
    public void setToneLevel(float toneLevel) {
        if (mLVRTCEngine != null) {
            mLVRTCEngine.getLVBeautyManager().setToneLevel(toneLevel);
        }
    }

    @Override
    public String getVersionName() {
        return LVRTCEngine.buildVersion();
    }
}
